rm (list=ls())
#modifying the current working directory
setwd("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/regression/tests_reg")
#loading the dataset
library(xlsReadWrite)
cars.data <- read.xls(file="cars_consumption.xls",colNames=T,sheet=1)
#regression
cars.model <-lm(consumption ~ ., data = cars.data)
print(cars.model)
#summary
detailed.model <- summary(cars.model)
print(detailed.model)
#sigma(error)
print(detailed.model$sigma)
#cov.unscaled matrix [X'X^(-1)]
print(detailed.model$cov.unscaled)

#**********************************
#test for simultaneous significance
#eng.size and horsepower
#**********************************

#estimed coefficient for eng.size and horsepower
a.test <- matrix(detailed.model$coefficients[2:3,1],nrow=2,ncol=1)
#reference values for comparison
ref.test <- matrix(c(0,0),nrow=2,ncol=1)
#difference between estimated coefficients and reference values
dif <- a.test - ref.test
#estimated covariance matrix of estimated coefficients
vcv.mat <- detailed.model$sigma^2 * detailed.model$cov.unscaled
#sub-covariance matrix for eng.size and horsepower
sub.vcv.mat <- vcv.mat[2:3,2:3]
print(sub.vcv.mat)
#inversion of the sub-matrix
inv.sub <- solve(sub.vcv.mat)
print(inv.sub)
#computing the statistical test
#the result is a scalar
F <- ((1/2)*(t(dif)%*%inv.sub%*%dif))[1,1]
print(F)
#computing the p-value
p <- 1.0-pf(F,2,24)
print(p)

#***************************
#test for linear combination
#eng.size and horsepower
#***************************

#estimated coefficient
a.hat <- matrix(detailed.model$coefficients[,1],nrow=4,ncol=1)
#R matrix !!! the intercept is in the first cell
R <- matrix(c(0,1000,-40,0),nrow=1,ncol=4)
#r vector
r <- matrix(c(0),nrow=1,ncol=1)
#R(X'X)^(-1)R'
B <- R%*%detailed.model$cov.unscaled%*%t(R)
#B^(-1)
inv.B <- solve(B)
#numerator of the statistic
F_num <- (1/1)*(t(R%*%a.hat-r)%*%inv.B%*%(R%*%a.hat-r))[1,1]
#denominator of the statistic
F_denom <- detailed.model$sigma^2
#F -statistic
F.stat <- F_num/F_denom
print(F.stat)
#p-value
p.value <- 1.0 - pf(F.stat,1,24)
print(p.value)

#**************************************
#prediction and its confidence interval
#**************************************

#description of the instance
x <- matrix(c(1,1984,85,1155),nrow=1,ncol=4)
#prediction
y.hat <- (x%*%a.hat)[1,1]
print(y.hat)
#leverage - it is a scalar value
lev <- (x%*%detailed.model$cov.unscaled%*%t(x))[1,1]
#variance of the error of prediction
v.err <- (detailed.model$sigma)^2 * (1 + lev)
#Student quantile for 0.95
tq <- qt(0.95,24)
#lower limit
y.1 <- y.hat - tq * sqrt(v.err)
print(y.1)
#upper limit
y.2 <- y.hat + tq * sqrt(v.err)
print(y.2)
